@@ -1,4 +1,3 @@
-#!/usr/bin/env python22
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """borg.py - We are the borg. Resistance is futile.
@@ -9,7 +8,7 @@ exist to encourage re-use, but I can not find any
 specific licencing terms.
 """
 
-from util import dbg
+from .util import dbg
 
 # pylint: disable-msg=R0903
 # pylint: disable-msg=R0921
@@ -17,7 +16,6 @@ class Borg:
     """Definition of a class that can never be duplicated. Correct usage is
     thus:
         
-    >>> from borg import Borg
     >>> class foo(Borg):
     ...     # All attributes on a borg class *must* = None
     ...     attribute = None
@@ -43,7 +41,7 @@ class Borg:
         type."""
         if borgtype is None:
             raise TypeError('Borg::__init__: You must pass a borgtype')
-        if not self.__shared_state.has_key(borgtype):
+        if borgtype not in self.__shared_state:
             dbg('Borg::__init__: Preparing borg state for %s' % borgtype)
             self.__shared_state[borgtype] = {}
         self.__dict__ = self.__shared_state[borgtype]
