@@ -73,7 +73,10 @@ KeyError: 'ConfigBase::get_item: unknown key algo'
 import os
 from copy import copy
 from configobj import ConfigObj, flatten_errors
-from validate import Validator
+try:
+    from validate import Validator
+except ModuleNotFoundError:
+    from configobj.validate import Validator
 from .borg import Borg
 from .util import dbg, err, DEBUG, get_config_dir, dict_diff
 
@@ -101,6 +104,7 @@ DEFAULTS = {
             'use_custom_url_handler': False,
             'custom_url_handler'    : '',
             'disable_real_transparency' : False,
+            'title_at_bottom'       : False,
             'title_hide_sizetext'   : False,
             'title_transmit_fg_color' : '#ffffff',
             'title_transmit_bg_color' : '#c80003',
@@ -117,7 +121,9 @@ DEFAULTS = {
             'title_use_system_font' : True,
             'title_font'            : 'Sans 9',
             'putty_paste_style'     : False,
+            'putty_paste_style_source_clipboard': False,
             'smart_copy'            : True,
+            'line_height'           : 1.0,
         },
         'keybindings': {
             'zoom_in'          : '<Control>plus',
@@ -172,6 +178,7 @@ DEFAULTS = {
             'reset'            : '<Shift><Control>r',
             'reset_clear'      : '<Shift><Control>g',
             'hide_window'      : '<Shift><Control><Alt>a',
+            'create_group'     : '',
             'group_all'        : '<Super>g',
             'group_all_toggle' : '',
             'ungroup_all'      : '<Shift><Super>g',
@@ -180,9 +187,9 @@ DEFAULTS = {
             'ungroup_tab'      : '<Shift><Super>t',
             'new_window'       : '<Shift><Control>i',
             'new_terminator'   : '<Super>i',
-            'broadcast_off'    : '<Alt>o',
-            'broadcast_group'  : '<Alt>g',
-            'broadcast_all'    : '<Alt>a',
+            'broadcast_off'    : '',
+            'broadcast_group'  : '',
+            'broadcast_all'    : '',
             'insert_number'    : '<Super>1',
             'insert_padded'    : '<Super>0',
             'edit_window_title': '<Control><Alt>w',
@@ -191,6 +198,7 @@ DEFAULTS = {
             'layout_launcher'  : '<Alt>l',
             'next_profile'     : '',
             'previous_profile' : '', 
+            'preferences'      : '',
             'help'             : 'F1'
         },
         'profiles': {
@@ -221,6 +229,7 @@ DEFAULTS = {
                 'scroll_on_output'      : False,
                 'scrollback_lines'      : 500,
                 'scrollback_infinite'   : False,
+                'disable_mousewheel_zoom': False,
                 'exit_action'           : 'close',
                 'palette'               : '#2e3436:#cc0000:#4e9a06:#c4a000:\
 #3465a4:#75507b:#06989a:#d3d7cf:#555753:#ef2929:#8ae234:#fce94f:\
@@ -232,6 +241,8 @@ DEFAULTS = {
                 'custom_command'        : '',
                 'use_system_font'       : True,
                 'use_theme_colors'      : False,
+                'bold_is_bright'        : False,
+                'line_height'           : 1.0,
                 'encoding'              : 'UTF-8',
                 'active_encodings'      : ['UTF-8', 'ISO-8859-1'],
                 'focus_on_close'        : 'auto',
@@ -243,6 +254,8 @@ DEFAULTS = {
                 'autoclean_groups'      : True,
                 'http_proxy'            : '',
                 'ignore_hosts'          : ['localhost','127.0.0.0/8','*.local'],
+                'background_image'      : '',
+                'background_alpha'      : 0.0
             },
         },
         'layouts': {
