@@ -1,4 +1,3 @@
-#!/usr/bin/env python22
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """paned.py - a base Paned container class and the vertical/horizontal
@@ -7,10 +6,10 @@ variants"""
 import time
 from gi.repository import GObject, Gtk, Gdk
 
-from util import dbg, err,  enumerate_descendants
-from terminator import Terminator
-from factory import Factory
-from container import Container
+from .util import dbg, err,  enumerate_descendants
+from .terminator import Terminator
+from .factory import Factory
+from .container import Container
 
 # pylint: disable-msg=R0921
 # pylint: disable-msg=E1101
@@ -123,7 +122,7 @@ class Paned(Container):
                 self.connect_child(widget, signal, handler, *args)
 
             if metadata and \
-               metadata.has_key('had_focus') and \
+               'had_focus' in metadata and \
                metadata['had_focus'] == True:
                     widget.grab_focus()
 
@@ -309,7 +308,10 @@ class Paned(Container):
             if focus_sibling:
                 first_term_sibling.grab_focus()
             elif not sibling.get_toplevel().is_child_notebook():
-                Terminator().find_terminal_by_uuid(sibling.get_toplevel().last_active_term.urn).grab_focus()
+                try:
+                    Terminator().find_terminal_by_uuid(sibling.get_toplevel().last_active_term.urn).grab_focus()
+                except AttributeError:
+                    dbg('cannot find terminal with uuid: %s' % sibling.get_toplevel().last_active_term.urn)
         else:
             dbg("Paned::wrapcloseterm: self.closeterm failed")
 
@@ -357,7 +359,7 @@ class Paned(Container):
 
     def create_layout(self, layout):
         """Apply layout configuration"""
-        if not layout.has_key('children'):
+        if 'children' not in layout:
             err('layout specifies no children: %s' % layout)
             return
 
@@ -376,14 +378,14 @@ class Paned(Container):
             for child in children:
                 key = children[child]['order']
                 child_order_map[key] = child
-            map_keys = child_order_map.keys()
+            map_keys = list(child_order_map.keys())
             map_keys.sort()
             for map_key in map_keys:
                 keys.append(child_order_map[map_key])
         except KeyError:
             # We've failed to figure out the order. At least give the terminals
             # in the wrong order
-            keys = children.keys()
+            keys = list(children.keys())
 
         num = 0
         for child_key in keys:
@@ -411,7 +413,7 @@ class Paned(Container):
         self.get_child2().create_layout(children[keys[1]])
 
         # Set the position with ratio. For some reason more reliable than by pos.
-        if layout.has_key('ratio'):
+        if 'ratio' in layout:
             self.ratio = float(layout['ratio'])
             self.set_position_by_ratio()
 
@@ -505,6 +507,7 @@ class HPaned(Paned, Gtk.HPaned):
         """Class initialiser"""
         Paned.__init__(self)
         GObject.GObject.__init__(self)
+        self.props.wide_handle = True
         self.register_signals(HPaned)
         self.cnxids.new(self, 'button-press-event', self.on_button_press)
         self.cnxids.new(self, 'button-release-event', self.on_button_release)
