@@ -1,31 +1,35 @@
-#!/usr/bin/env python22
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """terminal.py - classes necessary to provide Terminal widgets"""
 
-from __future__ import division
+
 import os
 import signal
 import gi
-from gi.repository import GLib, GObject, Pango, Gtk, Gdk
+from gi.repository import GLib, GObject, Pango, Gtk, Gdk, GdkPixbuf
 gi.require_version('Vte', '2.91')  # vte-0.38 (gnome-3.14)
 from gi.repository import Vte
 import subprocess
-import urllib
+try:
+    from urllib.parse import unquote as urlunquote
+except ImportError:
+    from urllib import unquote as urlunquote
 
-from util import dbg, err, spawn_new_terminator, make_uuid, manual_lookup, display_manager
-import util
-from config import Config
-from cwd import get_default_cwd
-from factory import Factory
-from terminator import Terminator
-from titlebar import Titlebar
-from terminal_popup_menu import TerminalPopupMenu
-from searchbar import Searchbar
-from translation import _
-from signalman import Signalman
-import plugin
+from .util import dbg, err, spawn_new_terminator, make_uuid, manual_lookup, display_manager
+from . import util
+from .config import Config
+from .cwd import get_pid_cwd
+from .factory import Factory
+from .terminator import Terminator
+from .titlebar import Titlebar
+from .terminal_popup_menu import TerminalPopupMenu
+from .prefseditor import PrefsEditor
+from .searchbar import Searchbar
+from .translation import _
+from .signalman import Signalman
+from . import plugin
 from terminatorlib.layoutlauncher import LayoutLauncher
+from . import regex
 
 # pylint: disable-msg=R0904
 class Terminal(Gtk.VBox):
@@ -63,7 +67,7 @@ class Terminal(Gtk.VBox):
             (GObject.TYPE_INT,)),
         'group-all': (GObject.SignalFlags.RUN_LAST, None, ()),
         'group-all-toggle': (GObject.SignalFlags.RUN_LAST, None, ()),
-        'move-tab': (GObject.SignalFlags.RUN_LAST, None, 
+        'move-tab': (GObject.SignalFlags.RUN_LAST, None,
             (GObject.TYPE_STRING,)),
     }
 
@@ -72,7 +76,7 @@ class Terminal(Gtk.VBox):
 
     MOUSEBUTTON_LEFT = 1
     MOUSEBUTTON_MIDDLE = 2
-    MOUSEBUTTON_RIGHT = 3    
+    MOUSEBUTTON_RIGHT = 3
 
     terminator = None
     vte = None
@@ -125,13 +129,25 @@ class Terminal(Gtk.VBox):
 
         self.config = Config()
 
-        self.cwd = get_default_cwd()
+        self.cwd = get_pid_cwd()
         self.origcwd = self.terminator.origcwd
         self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
 
         self.pending_on_vte_size_allocate = False
 
         self.vte = Vte.Terminal()
+        self.background_image = None
+        if self.config['background_image'] != '':
+            self.vte.set_clear_background(False)
+            self.vte.connect("draw",self.background_draw)
+
+            try: 
+                self.background_image = GdkPixbuf.Pixbuf.new_from_file(self.config['background_image'])
+            except Exception:
+                pass
+
+        self.background_alpha = self.config['background_alpha']
+        self.vte.set_allow_hyperlink(True)
         self.vte._draw_data = None
         if not hasattr(self.vte, "set_opacity") or \
            not hasattr(self.vte, "is_composited"):
@@ -140,11 +156,9 @@ class Terminal(Gtk.VBox):
             self.composite_support = True
         dbg('composite_support: %s' % self.composite_support)
 
+        
         self.vte.show()
-
         self.default_encoding = self.vte.get_encoding()
-        self.regex_flags = (GLib.RegexCompileFlags.OPTIMIZE | \
-                            GLib.RegexCompileFlags.MULTILINE)
         self.update_url_matches()
 
         self.terminalbox = self.create_terminalbox()
@@ -160,8 +174,13 @@ class Terminal(Gtk.VBox):
         self.searchbar.connect('end-search', self.on_search_done)
 
         self.show()
-        self.pack_start(self.titlebar, False, True, 0)
-        self.pack_start(self.terminalbox, True, True, 0)
+        if self.config['title_at_bottom']:
+            self.pack_start(self.terminalbox, True, True, 0)
+            self.pack_start(self.titlebar, False, True, 0)
+        else:
+            self.pack_start(self.titlebar, False, True, 0)
+            self.pack_start(self.terminalbox, True, True, 0)
+
         self.pack_end(self.searchbar, True, True, 0)
 
         self.connect_signals()
@@ -222,7 +241,8 @@ class Terminal(Gtk.VBox):
             return(GLib.filename_from_uri(vte_cwd)[0])
         else:
             # Fall back to old gtk2 method
-            return(self.terminator.pid_cwd(self.pid))
+            dbg('calling get_pid_cwd')
+            return(get_pid_cwd(self.pid))
 
     def close(self):
         """Close ourselves"""
@@ -232,7 +252,7 @@ class Terminal(Gtk.VBox):
         try:
             dbg('close: killing %d' % self.pid)
             os.kill(self.pid, signal.SIGHUP)
-        except Exception, ex:
+        except Exception as ex:
             # We really don't want to care if this failed. Deep OS voodoo is
             # not what we should be doing.
             dbg('os.kill failed: %s' % ex)
@@ -255,6 +275,24 @@ class Terminal(Gtk.VBox):
 
         return(terminalbox)
 
+    def _add_regex(self, name, re):
+        match = -1
+        if regex.FLAGS_PCRE2:
+            try:
+                reg = Vte.Regex.new_for_match(re, len(re), self.regex_flags or regex.FLAGS_PCRE2)
+                match = self.vte.match_add_regex(reg, 0)
+            except GLib.Error:
+                # happens when PCRE2 support is not builtin (Ubuntu < 19.10)
+                pass
+
+        # try the "old" glib regex
+        if match < 0:
+            reg = GLib.Regex.new(re, self.regex_flags or regex.FLAGS_GLIB, 0)
+            match = self.vte.match_add_gregex(reg, 0)
+
+        self.matches[name] = match
+        self.vte.match_set_cursor_name(self.matches[name], 'pointer')
+
     def update_url_matches(self):
         """Update the regexps used to match URLs"""
         userchars = "-A-Za-z0-9"
@@ -269,36 +307,34 @@ class Terminal(Gtk.VBox):
         rboundry = "\\b"
 
         re = (lboundry + schemes +
-                "//(" + user + "@)?[" + hostchars  +".]+(:[0-9]+)?(" + 
+                "//(" + user + "@)?[" + hostchars  +".]+(:[0-9]+)?(" +
                 urlpath + ")?" + rboundry + "/?")
-        reg = GLib.Regex.new(re, self.regex_flags, 0)
-        self.matches['full_uri'] = self.vte.match_add_gregex(reg, 0)
+        self._add_regex('full_uri', re)
 
         if self.matches['full_uri'] == -1:
             err ('Terminal::update_url_matches: Failed adding URL matches')
         else:
             re = (lboundry +
-                    '(callto:|h323:|sip:)' + "[" + userchars + "+][" + 
-                    userchars + ".]*(:[0-9]+)?@?[" + pathchars + "]+" + 
+                    '(callto:|h323:|sip:)' + "[" + userchars + "+][" +
+                    userchars + ".]*(:[0-9]+)?@?[" + pathchars + "]+" +
                     rboundry)
-            reg = GLib.Regex.new(re, self.regex_flags, 0)
-            self.matches['voip'] = self.vte.match_add_gregex(reg, 0)
+            self._add_regex('voip', re)
+
             re = (lboundry +
-                    "(www|ftp)[" + hostchars + "]*\.[" + hostchars + 
+                    "(www|ftp)[" + hostchars + "]*\.[" + hostchars +
                     ".]+(:[0-9]+)?(" + urlpath + ")?" + rboundry + "/?")
-            reg = GLib.Regex.new(re, self.regex_flags, 0)
-            self.matches['addr_only'] = self.vte.match_add_gregex(reg, 0)
+            self._add_regex('addr_only', re)
+
             re = (lboundry +
                     "(mailto:)?[a-zA-Z0-9][a-zA-Z0-9.+-]*@[a-zA-Z0-9]" +
                             "[a-zA-Z0-9-]*\.[a-zA-Z0-9][a-zA-Z0-9-]+" +
                             "[.a-zA-Z0-9-]*" + rboundry)
-            reg = GLib.Regex.new(re, self.regex_flags, 0)
-            self.matches['email'] = self.vte.match_add_gregex(reg, 0)
+            self._add_regex('email', re)
+
             re = (lboundry +
                   """news:[-A-Z\^_a-z{|}~!"#$%&'()*+,./0-9;:=?`]+@""" +
                             "[-A-Za-z0-9.]+(:[0-9]+)?" + rboundry)
-            reg = GLib.Regex.new(re, self.regex_flags, 0)
-            self.matches['nntp'] = self.vte.match_add_gregex(reg, 0)
+            self._add_regex('nntp', re)
 
             # Now add any matches from plugins
             try:
@@ -312,12 +348,13 @@ class Terminal(Gtk.VBox):
                     if name in self.matches:
                         dbg('refusing to add duplicate match %s' % name)
                         continue
-                    reg = GLib.Regex.new(match, self.regex_flags, 0)
-                    self.matches[name] = self.vte.match_add_gregex(reg, 0)
-                    dbg('added plugin URL handler for %s (%s) as %d' % 
+
+                    self._add_regex(name, match)
+
+                    dbg('added plugin URL handler for %s (%s) as %d' %
                         (name, urlplugin.__class__.__name__,
                         self.matches[name]))
-            except Exception, ex:
+            except Exception as ex:
                 err('Exception occurred adding plugin URL match: %s' % ex)
 
     def match_add(self, name, match):
@@ -325,8 +362,8 @@ class Terminal(Gtk.VBox):
         if name in self.matches:
             err('Terminal::match_add: Refusing to create duplicate match %s' % name)
             return
-        reg = GLib.Regex.new(match, self.regex_flags, 0)
-        self.matches[name] = self.vte.match_add_gregex(reg, 0)
+
+        self._add_regex(name, match)
 
     def match_remove(self, name):
         """Remove a previously registered URL match"""
@@ -351,8 +388,8 @@ class Terminal(Gtk.VBox):
         self.cnxids.new(self.vte, 'popup-menu', self.popup_menu)
 
         srcvtetargets = [("vte", Gtk.TargetFlags.SAME_APP, self.TARGET_TYPE_VTE)]
-        dsttargets = [("vte", Gtk.TargetFlags.SAME_APP, self.TARGET_TYPE_VTE), 
-                      ('text/x-moz-url', 0, self.TARGET_TYPE_MOZ), 
+        dsttargets = [("vte", Gtk.TargetFlags.SAME_APP, self.TARGET_TYPE_VTE),
+                      ('text/x-moz-url', 0, self.TARGET_TYPE_MOZ),
                       ('_NETSCAPE_URL', 0, 0)]
         '''
         The following should work, but on my system it corrupts the returned
@@ -381,7 +418,7 @@ class Terminal(Gtk.VBox):
         dbg('Finalised drag targets: %s' % dsttargets)
 
         for (widget, mask) in [
-            (self.vte, Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.BUTTON3_MASK), 
+            (self.vte, Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.BUTTON3_MASK),
             (self.titlebar, Gdk.ModifierType.BUTTON1_MASK)]:
             widget.drag_source_set(mask, srcvtetargets, Gdk.DragAction.MOVE)
 
@@ -398,7 +435,7 @@ class Terminal(Gtk.VBox):
         self.cnxids.new(self.vte, 'drag-data-received',
             self.on_drag_data_received, self)
 
-        self.cnxids.new(self.vte, 'selection-changed', 
+        self.cnxids.new(self.vte, 'selection-changed',
             lambda widget: self.maybe_copy_clipboard())
 
         if self.composite_support:
@@ -495,9 +532,9 @@ class Terminal(Gtk.VBox):
         groupitems = []
         cnxs = []
 
-        for key, value in {_('Broadcast _all'):'all', 
+        for key, value in list({_('Broadcast _all'):'all',
                           _('Broadcast _group'):'group',
-                          _('Broadcast _off'):'off'}.items():
+                          _('Broadcast _off'):'off'}.items()):
             item = Gtk.RadioMenuItem.new_with_mnemonic(groupitems, key)
             groupitems = item.get_group()
             dbg('Terminal::populate_group_menu: %s active: %s' %
@@ -588,7 +625,7 @@ class Terminal(Gtk.VBox):
     def set_groupsend(self, _widget, value):
         """Set the groupsend mode"""
         # FIXME: Can we think of a smarter way of doing this than poking?
-        if value in self.terminator.groupsend_type.values():
+        if value in list(self.terminator.groupsend_type.values()):
             dbg('Terminal::set_groupsend: setting groupsend to %s' % value)
             self.terminator.groupsend = value
 
@@ -611,7 +648,7 @@ class Terminal(Gtk.VBox):
         if self.config['exit_action'] == 'restart':
             self.cnxids.new(self.vte, 'child-exited', self.spawn_child, True)
         elif self.config['exit_action'] in ('close', 'left'):
-            self.cnxids.new(self.vte, 'child-exited', 
+            self.cnxids.new(self.vte, 'child-exited',
                                             lambda x, y: self.emit('close-term'))
 
         if self.custom_encoding != True:
@@ -675,6 +712,11 @@ class Terminal(Gtk.VBox):
             except:
                 pass
         self.vte.set_allow_bold(self.config['allow_bold'])
+        if hasattr(self.vte,'set_cell_height_scale'): 
+            self.vte.set_cell_height_scale(self.config['line_height'])
+        if hasattr(self.vte, 'set_bold_is_bright'):
+            self.vte.set_bold_is_bright(self.config['bold_is_bright'])
+
         if self.config['use_theme_colors']:
             self.fgcolor_active = self.vte.get_style_context().get_color(Gtk.StateType.NORMAL)  # VERIFY FOR GTK3: do these really take the theme colors?
             self.bgcolor = self.vte.get_style_context().get_background_color(Gtk.StateType.NORMAL)
@@ -714,20 +756,20 @@ class Terminal(Gtk.VBox):
         if len(colors) == 16:
             # RGB values for indices 16..255 copied from vte source in order to dim them
             shades = [0, 95, 135, 175, 215, 255]
-            for r in xrange(0, 6):
-                for g in xrange(0, 6):
-                    for b in xrange(0, 6):
+            for r in range(0, 6):
+                for g in range(0, 6):
+                    for b in range(0, 6):
                         newcolor = Gdk.RGBA()
                         setattr(newcolor, "red",   shades[r] / 255.0)
                         setattr(newcolor, "green", shades[g] / 255.0)
                         setattr(newcolor, "blue",  shades[b] / 255.0)
                         self.palette_active.append(newcolor)
-            for y in xrange(8, 248, 10):
+            for y in range(8, 248, 10):
                 newcolor = Gdk.RGBA()
                 setattr(newcolor, "red",   y / 255.0)
                 setattr(newcolor, "green", y / 255.0)
                 setattr(newcolor, "blue",  y / 255.0)
-                self.palette_active.append(newcolor)        
+                self.palette_active.append(newcolor)
         self.palette_inactive = []
         for color in self.palette_active:
             newcolor = Gdk.RGBA()
@@ -743,7 +785,7 @@ class Terminal(Gtk.VBox):
                                 self.palette_inactive)
         profiles = self.config.base.profiles
         terminal_box_style_context = self.terminalbox.get_style_context()
-        for profile in profiles.keys():
+        for profile in list(profiles.keys()):
             munged_profile = "terminator-profile-%s" % (
                 "".join([c if c.isalnum() else "-" for c in profile]))
             if terminal_box_style_context.has_class(munged_profile):
@@ -791,6 +833,7 @@ class Terminal(Gtk.VBox):
             elif self.config['scrollbar_position'] == 'right':
                 self.terminalbox.reorder_child(self.vte, 0)
 
+        
         self.vte.set_rewrap_on_resize(self.config['rewrap_on_resize'])
 
         self.titlebar.update()
@@ -799,7 +842,7 @@ class Terminal(Gtk.VBox):
     def set_cursor_color(self):
         """Set the cursor color appropriately"""
         if self.config['cursor_color_fg']:
-            self.vte.set_color_cursor(None) 
+            self.vte.set_color_cursor(None)
         else:
             cursor_color = Gdk.RGBA()
             cursor_color.parse(self.config['cursor_color'])
@@ -842,6 +885,8 @@ class Terminal(Gtk.VBox):
             elif event.type == Gdk.EventType.BUTTON_PRESS:
                 # Single Click gives popup
                 dbg('on_group_button_press: group menu popup')
+                window = self.get_toplevel()
+                window.preventHide = True
                 self.create_popup_group_menu(widget, event)
                 return True
             else:
@@ -865,10 +910,24 @@ class Terminal(Gtk.VBox):
         # FIXME: Does keybindings really want to live in Terminator()?
         mapping = self.terminator.keybindings.lookup(event)
 
+        # Just propagate tab-swictch events if there is only one tab
+        if (
+                mapping and (
+                    mapping.startswith('switch_to_tab') or
+                    mapping in ('next_tab', 'prev_tab')
+                )
+        ):
+            window = self.get_toplevel()
+            child = window.get_children()[0]
+            if isinstance(child, Terminal):
+                # not a Notebook instance => a single tab is used
+                # .get_n_pages() can not be used
+                return(False)
+
         if mapping == "hide_window":
             return(False)
 
-        if mapping and mapping not in ['close_window', 
+        if mapping and mapping not in ['close_window',
                                        'full_screen']:
             dbg('Terminal::on_keypress: lookup found: %r' % mapping)
             # handle the case where user has re-bound copy to ctrl+<key>
@@ -907,27 +966,38 @@ class Terminal(Gtk.VBox):
             # Suppress double-click behavior
             return True
 
-        use_primary = (display_manager() != 'WAYLAND')
         if self.config['putty_paste_style']:
             middle_click = [self.popup_menu, (widget, event)]
-            right_click = [self.paste_clipboard, (use_primary, )]
+            right_click = [self.paste_clipboard, (not self.config['putty_paste_style_source_clipboard'], )]
         else:
-            middle_click = [self.paste_clipboard, (use_primary, )]
+            middle_click = [self.paste_clipboard, (True, )]
             right_click = [self.popup_menu, (widget, event)]
 
+        ### Ctrl-click event here.
         if event.button == self.MOUSEBUTTON_LEFT:
             # Ctrl+leftclick on a URL should open it
             if event.get_state() & Gdk.ModifierType.CONTROL_MASK == Gdk.ModifierType.CONTROL_MASK:
-                url = self.vte.match_check_event(event)
-                if url[0]:
-                    self.open_url(url, prepare=True)
+                ## Check new OSC-8 method first
+                url = self.vte.hyperlink_check_event(event)
+                dbg('url: %s' % url)
+                if url:
+                    self.open_url(url, prepare=False)
+                else:
+                    dbg('OSC-8 URL not detected dropping back to regex match')
+                    url = self.vte.match_check_event(event)
+                    if url[0]:
+                        self.open_url(url, prepare=True)
         elif event.button == self.MOUSEBUTTON_MIDDLE:
             # middleclick should paste the clipboard
             # try to pass it to vte widget first though
             if event.get_state() & Gdk.ModifierType.CONTROL_MASK == 0:
                 if event.get_state() & Gdk.ModifierType.SHIFT_MASK == 0:
+                    gtk_settings=Gtk.Settings().get_default()
+                    primary_state = gtk_settings.get_property('gtk-enable-primary-paste')
+                    gtk_settings.set_property('gtk-enable-primary-paste',  False)
                     if not Vte.Terminal.do_button_press_event(self.vte, event):
                         middle_click[0](*middle_click[1])
+                    gtk_settings.set_property('gtk-enable-primary-paste', primary_state)
                 else:
                     middle_click[0](*middle_click[1])
                 return(True)
@@ -945,13 +1015,16 @@ class Terminal(Gtk.VBox):
                 return(True)
 
         return(False)
-    
+
     def on_mousewheel(self, widget, event):
         """Handler for modifier + mouse wheel scroll events"""
         SMOOTH_SCROLL_UP = event.direction == Gdk.ScrollDirection.SMOOTH and event.delta_y <= 0.
         SMOOTH_SCROLL_DOWN = event.direction == Gdk.ScrollDirection.SMOOTH and event.delta_y > 0.
         if event.state & Gdk.ModifierType.CONTROL_MASK == Gdk.ModifierType.CONTROL_MASK:
-            # Ctrl + mouse wheel up/down with Shift and Super additions
+            # Zoom the terminal(s) in or out if not disabled in config
+            if self.config["disable_mousewheel_zoom"] is True:
+                return (False)
+            # Choice of target terminals depends on Shift and Super modifiers
             if event.state & Gdk.ModifierType.MOD4_MASK == Gdk.ModifierType.MOD4_MASK:
                 targets=self.terminator.terminals
             elif event.state & Gdk.ModifierType.SHIFT_MASK == Gdk.ModifierType.SHIFT_MASK:
@@ -978,6 +1051,8 @@ class Terminal(Gtk.VBox):
 
     def popup_menu(self, widget, event=None):
         """Display the context menu"""
+        window = self.get_toplevel()
+        window.preventHide = True
         menu = TerminalPopupMenu(self)
         menu.show(widget, event)
 
@@ -1004,11 +1079,11 @@ class Terminal(Gtk.VBox):
         """Handle the start of a drag event"""
         Gtk.drag_set_icon_pixbuf(drag_context, util.widget_pixbuf(self, 512), 0, 0)
 
-    def on_drag_data_get(self, _widget, _drag_context, selection_data, info, 
+    def on_drag_data_get(self, _widget, _drag_context, selection_data, info,
             _time, data):
         """I have no idea what this does, drag and drop is a mystery. sorry."""
         selection_data.set(Gdk.atom_intern('vte', False), info,
-                str(data.terminator.terminals.index(self)))
+                bytes(str(data.terminator.terminals.index(self)), 'utf-8'))
 
     def on_drag_motion(self, widget, drag_context, x, y, _time, _data):
         """*shrug*"""
@@ -1018,7 +1093,7 @@ class Terminal(Gtk.VBox):
             # copy text from another widget
             return
         srcwidget = Gtk.drag_get_source_widget(drag_context)
-        if(isinstance(srcwidget, Gtk.EventBox) and 
+        if(isinstance(srcwidget, Gtk.EventBox) and
            srcwidget == self.titlebar) or widget == srcwidget:
             # on self
             return
@@ -1049,7 +1124,7 @@ class Terminal(Gtk.VBox):
         elif pos == "left":
             coord = (topleft, topmiddle, bottommiddle, bottomleft)
         elif pos == "bottom":
-            coord = (bottomleft, bottomright, middleright , middleleft) 
+            coord = (bottomleft, bottomright, middleright , middleleft)
 
         #here, we define some widget internal values
         widget._draw_data = { 'color': color, 'coord' : coord }
@@ -1061,8 +1136,23 @@ class Terminal(Gtk.VBox):
         widget.disconnect(connec)
         widget._draw_data = None
 
+    def background_draw(self, widget, cr):
+        if not self.background_image:
+                return(False)
+        over = self.bgcolor
+        over.alpha = self.background_alpha
+        rect = self.vte.get_allocation()
+        xratio = float(rect.width) / float(self.background_image.get_width())
+        yratio = float(rect.height) / float(self.background_image.get_height())
+        cr.save()
+        cr.scale(xratio,yratio)
+        Gdk.cairo_set_source_pixbuf(cr, self.background_image, 0, 0)
+        cr.paint()
+        Gdk.cairo_set_source_rgba(cr,over)
+        cr.paint()
+        cr.restore()
+
     def on_draw(self, widget, context):
-        """Handle an expose event while dragging"""
         if not widget._draw_data:
             return(False)
 
@@ -1090,8 +1180,10 @@ class Terminal(Gtk.VBox):
 
             # https://bugs.launchpad.net/terminator/+bug/1518705
             if info == self.TARGET_TYPE_MOZ:
-                 txt = txt.decode('utf-16').encode('utf-8')
+                 txt = txt.decode('utf-16')
                  txt = txt.split('\n')[0]
+            else:
+                 txt = txt.decode()
 
             txt_lines = txt.split( "\r\n" )
             if txt_lines[-1] == '':
@@ -1105,18 +1197,18 @@ class Terminal(Gtk.VBox):
                     str=''
                     for fname in txt_lines[:-1]:
                         dbg('drag data fname: %s' % fname)
-                        fname = "'%s'" % urllib.unquote(fname[7:].replace("'",
+                        fname = "'%s'" % urlunquote(fname[7:].replace("'",
                                                                     '\'\\\'\''))
                         str += fname + ' '
                     txt=str
             for term in self.terminator.get_target_terms(self):
                 term.feed(txt)
             return
-        
+
         widgetsrc = data.terminator.terminals[int(selection_data.get_data())]
         srcvte = Gtk.drag_get_source_widget(drag_context)
         #check if computation requireds
-        if (isinstance(srcvte, Gtk.EventBox) and 
+        if (isinstance(srcvte, Gtk.EventBox) and
                 srcvte == self.titlebar) or srcvte == widget:
             return
 
@@ -1182,7 +1274,11 @@ class Terminal(Gtk.VBox):
     def ensure_visible_and_focussed(self):
         """Make sure that we're visible and focussed"""
         window = self.get_toplevel()
-        topchild = window.get_children()[0]
+        try:
+            topchild = window.get_children()[0]
+        except IndexError:
+            dbg('unable to get top child')    
+            return
         maker = Factory()
 
         if maker.isinstance(topchild, 'Notebook'):
@@ -1390,7 +1486,7 @@ class Terminal(Gtk.VBox):
             shell = util.shell_lookup()
 
             if self.config['login_shell']:
-                args.insert(0, "-%s" % shell)
+                args.insert(0, "-l")
             else:
                 args.insert(0, shell)
 
@@ -1445,7 +1541,7 @@ class Terminal(Gtk.VBox):
             url = 'ftp://' + url
         elif match == self.matches['addr_only']:
             url = 'http://' + url
-        elif match in self.matches.values():
+        elif match in list(self.matches.values()):
             # We have a match, but it's not a hard coded one, so it's a plugin
             try:
                 registry = plugin.PluginRegistry()
@@ -1460,7 +1556,7 @@ class Terminal(Gtk.VBox):
 %s plugin' % urlplugin.handler_name)
                             url = newurl
                         break
-            except Exception, ex:
+            except Exception as ex:
                 err('Exception occurred preparing URL: %s' % ex)
 
         return(url)
@@ -1504,7 +1600,7 @@ class Terminal(Gtk.VBox):
 
     def feed(self, text):
         """Feed the supplied text to VTE"""
-        self.vte.feed_child(text, len(text))
+        self.vte.feed_child_binary(text.encode(self.vte.get_encoding()))
 
     def zoom_in(self):
         """Increase the font size"""
@@ -1621,20 +1717,20 @@ class Terminal(Gtk.VBox):
     def create_layout(self, layout):
         """Apply our layout"""
         dbg('Setting layout')
-        if layout.has_key('command') and layout['command'] != '':
+        if 'command' in layout and layout['command'] != '':
             self.layout_command = layout['command']
-        if layout.has_key('profile') and layout['profile'] != '':
+        if 'profile' in layout and layout['profile'] != '':
             if layout['profile'] in self.config.list_profiles():
                 self.set_profile(self, layout['profile'])
-        if layout.has_key('group') and layout['group'] != '':
+        if 'group' in layout and layout['group'] != '':
             # This doesn't need/use self.titlebar, but it's safer than sending
             # None
             self.really_create_group(self.titlebar, layout['group'])
-        if layout.has_key('title') and layout['title'] != '':
+        if 'title' in layout and layout['title'] != '':
             self.titlebar.set_custom_string(layout['title'])
-        if layout.has_key('directory') and layout['directory'] != '':
+        if 'directory' in layout and layout['directory'] != '':
             self.directory = layout['directory']
-        if layout.has_key('uuid') and layout['uuid'] != '':
+        if 'uuid' in layout and layout['uuid'] != '':
             self.uuid = make_uuid(layout['uuid'])
 
     def scroll_by_page(self, pages):
@@ -1805,6 +1901,9 @@ class Terminal(Gtk.VBox):
     def key_reset_clear(self):
         self.vte.reset (True, True)
 
+    def key_create_group(self):
+        self.titlebar.create_group()
+
     def key_group_all(self):
         self.emit('group-all')
 
@@ -1846,7 +1945,7 @@ class Terminal(Gtk.VBox):
 
     def key_insert_number(self):
         self.emit('enumerate', False)
-    
+
     def key_insert_padded(self):
         self.emit('enumerate', True)
 
@@ -1859,13 +1958,13 @@ class Terminal(Gtk.VBox):
         dialog.set_default_response(Gtk.ResponseType.ACCEPT)
         dialog.set_resizable(False)
         dialog.set_border_width(8)
-        
+
         label = Gtk.Label(label=_('Enter a new title for the Terminator window...'))
         name = Gtk.Entry()
         name.set_activates_default(True)
         if window.title.text != self.vte.get_window_title():
             name.set_text(self.get_toplevel().title.text)
-        
+
         dialog.vbox.pack_start(label, False, False, 6)
         dialog.vbox.pack_start(name, False, False, 6)
 
@@ -1915,6 +2014,9 @@ class Terminal(Gtk.VBox):
     def key_line_down(self):
         self.scroll_by_line(1)
 
+    def key_preferences(self):
+        PrefsEditor(self)
+
     def key_help(self):
         manual_index_page = manual_lookup()
         if manual_index_page:
