@@ -1,4 +1,3 @@
-#!/usr/bin/env python22
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """terminator.py - class for the master Terminator singleton"""
@@ -7,17 +6,22 @@ import copy
 import os
 import gi
 gi.require_version('Vte', '2.91')
-from gi.repository import Gtk, Gdk, Vte, GdkX11
+from gi.repository import Gtk, Gdk, Vte
 from gi.repository.GLib import GError
 
-import borg
-from borg import Borg
-from config import Config
-from keybindings import Keybindings
-from util import dbg, err, enumerate_descendants
-from factory import Factory
-from cwd import get_pid_cwd
-from version import APP_NAME, APP_VERSION
+from . import borg
+from .borg import Borg
+from .config import Config
+from .keybindings import Keybindings
+from .util import dbg, err, enumerate_descendants
+from .factory import Factory
+from .version import APP_NAME, APP_VERSION
+
+try:
+    from gi.repository import GdkX11
+except ImportError:
+    dbg("could not import X11 gir module")
+
 
 def eventkey2gdkevent(eventkey):  # FIXME FOR GTK3: is there a simpler way of casting from specific EventKey to generic (union) GdkEvent?
     gdkevent = Gdk.Event.new(eventkey.type)
@@ -49,8 +53,6 @@ class Terminator(Borg):
     origcwd = None
     dbus_path = None
     dbus_name = None
-    pid_cwd = None
-    gnome_client = None
     debug_address = None
     ibus_running = None
 
@@ -93,10 +95,6 @@ class Terminator(Borg):
             self.style_providers = []
         if not self.doing_layout:
             self.doing_layout = False
-        if not self.pid_cwd:
-            self.pid_cwd = get_pid_cwd()
-        if self.gnome_client is None:
-            self.attempt_gnome_client()
         self.connect_signals()
 
     def connect_signals(self):
@@ -118,30 +116,6 @@ class Terminator(Borg):
             self.dbus_name = dbus_service.bus_name.get_name()
             self.dbus_path = dbus_service.bus_path
 
-    def attempt_gnome_client(self):
-        """Attempt to find a GNOME Session to register with"""
-        try:
-            from gi.repository import Gnome
-            self.gnome_program = Gnome.init(APP_NAME, APP_VERSION)  # VERIFY FOR GTK3
-            self.gnome_client = Gnome.Ui.master_client()  # VERIFY FOR GTK3
-            self.gnome_client.connect_to_session_manager()
-            self.gnome_client.connect('save-yourself', self.save_yourself)
-            self.gnome_client.connect('die', self.die)
-            dbg('GNOME session support enabled and registered')
-        except (ImportError, AttributeError):
-            self.gnome_client = False
-            dbg('GNOME session support not available')
-
-    def save_yourself(self, *args):
-        """Save as much state as possible for the session manager"""
-        dbg('preparing session manager state')
-        # FIXME: Implement this
-
-    def die(self, *args):
-        """Die at the hands of the session manager"""
-        dbg('session manager asked us to die')
-        # FIXME: Implement this
-
     def get_windows(self):
         """Return a list of windows"""
         return self.windows
@@ -267,34 +241,34 @@ class Terminator(Borg):
             count = count + 1
             if count == 1000:
                 err('hit maximum loop boundary. THIS IS VERY LIKELY A BUG')
-            for obj in layout.keys():
+            for obj in list(layout.keys()):
                 if layout[obj]['type'].lower() == 'window':
                     hierarchy[obj] = {}
                     hierarchy[obj]['type'] = 'Window'
                     hierarchy[obj]['children'] = {}
 
                     # Copy any additional keys
-                    for objkey in layout[obj].keys():
-                        if layout[obj][objkey] != '' and not hierarchy[obj].has_key(objkey):
+                    for objkey in list(layout[obj].keys()):
+                        if layout[obj][objkey] != '' and objkey not in hierarchy[obj]:
                             hierarchy[obj][objkey] = layout[obj][objkey]
 
                     objects[obj] = hierarchy[obj]
                     del(layout[obj])
                 else:
                     # Now examine children to see if their parents exist yet
-                    if not layout[obj].has_key('parent'):
+                    if 'parent' not in layout[obj]:
                         err('Invalid object: %s' % obj)
                         del(layout[obj])
                         continue
-                    if objects.has_key(layout[obj]['parent']):
+                    if layout[obj]['parent'] in objects:
                         # Our parent has been created, add ourselves
                         childobj = {}
                         childobj['type'] = layout[obj]['type']
                         childobj['children'] = {}
 
                         # Copy over any additional object keys
-                        for objkey in layout[obj].keys():
-                            if not childobj.has_key(objkey):
+                        for objkey in list(layout[obj].keys()):
+                            if objkey not in childobj:
                                 childobj[objkey] = layout[obj][objkey]
 
                         objects[layout[obj]['parent']]['children'][obj] = childobj
@@ -309,25 +283,25 @@ class Terminator(Borg):
                 raise(ValueError)
             dbg('Creating a window')
             window, terminal = self.new_window()
-            if layout[windef].has_key('position'):
+            if 'position' in layout[windef]:
                 parts = layout[windef]['position'].split(':')
                 if len(parts) == 2:
                     window.move(int(parts[0]), int(parts[1]))
-            if layout[windef].has_key('size'):
+            if 'size' in layout[windef]:
                 parts = layout[windef]['size']
                 winx = int(parts[0])
                 winy = int(parts[1])
                 if winx > 1 and winy > 1:
                     window.resize(winx, winy)
-            if layout[windef].has_key('title'):
+            if 'title' in layout[windef]:
                 window.title.force_title(layout[windef]['title'])
-            if layout[windef].has_key('maximised'):
+            if 'maximised' in layout[windef]:
                 if layout[windef]['maximised'] == 'True':
                     window.ismaximised = True
                 else:
                     window.ismaximised = False
                 window.set_maximised(window.ismaximised)
-            if layout[windef].has_key('fullscreen'):
+            if 'fullscreen' in layout[windef]:
                 if layout[windef]['fullscreen'] == 'True':
                     window.isfullscreen = True
                 else:
@@ -355,35 +329,7 @@ class Terminator(Borg):
                 terminal.spawn_child()
 
         for window in self.windows:
-            if window.is_child_notebook():
-                # For windows with a notebook
-                notebook = window.get_toplevel().get_children()[0]
-                # Cycle through pages by number
-                for page in xrange(0, notebook.get_n_pages()):
-                    # Try and get the entry in the previously saved mapping
-                    mapping = window_last_active_term_mapping[window]
-                    page_last_active_term = mapping.get(notebook.get_nth_page(page),  None)
-                    if page_last_active_term is None:
-                        # Couldn't find entry, so we find the first child of type Terminal
-                        children = notebook.get_nth_page(page).get_children()
-                        for page_last_active_term in children:
-                            if maker.isinstance(page_last_active_term, 'Terminal'):
-                                page_last_active_term = page_last_active_term.uuid
-                                break
-                        else:
-                            err('Should never reach here!')
-                            page_last_active_term = None
-                    if page_last_active_term is None:
-                        # Bail on this tab as we're having no luck here, continue with the next
-                        continue
-                    # Set the notebook entry, then ensure Terminal is visible and focussed
-                    urn = page_last_active_term.urn
-                    notebook.last_active_term[notebook.get_nth_page(page)] = page_last_active_term
-                    if urn:
-                        term = self.find_terminal_by_uuid(urn)
-                        if term:
-                            term.ensure_visible_and_focussed()
-            else:
+            if not window.is_child_notebook():
                 # For windows without a notebook ensure Terminal is visible and focussed
                 if window_last_active_term_mapping[window]:
                     term = self.find_terminal_by_uuid(window_last_active_term_mapping[window].urn)
@@ -391,17 +337,18 @@ class Terminator(Borg):
 
         # Build list of new windows using prelayout list
         new_win_list = []
-        for window in self.windows:
-            if window not in self.prelayout_windows:
-                new_win_list.append(window)
-        
+        if self.prelayout_windows:
+            for window in self.windows:
+                if window not in self.prelayout_windows:
+                    new_win_list.append(window)
+
         # Make sure all new windows get bumped to the top
         for window in new_win_list:
             window.show()
             window.grab_focus()
             try:
                 t = GdkX11.x11_get_server_time(window.get_window())
-            except (TypeError, AttributeError):
+            except (NameError,TypeError, AttributeError):
                 t = 0
             window.get_window().focus(t)
 
@@ -417,7 +364,7 @@ class Terminator(Borg):
                 window.grab_focus()
                 try:
                     t = GdkX11.x11_get_server_time(window.get_window())
-                except (TypeError, AttributeError):
+                except (NameError,TypeError, AttributeError):
                     t = 0
                 window.get_window().focus(t)
 
@@ -474,7 +421,7 @@ class Terminator(Borg):
                 background-color: alpha(%s, %s); }
             """
         profiles = self.config.base.profiles
-        for profile in profiles.keys():
+        for profile in list(profiles.keys()):
             if profiles[profile]['use_theme_colors']:
                 # Create a dummy window/vte and realise it so it has correct
                 # values to read from
@@ -501,7 +448,7 @@ class Terminator(Borg):
             css += template % (munged_profile, bgcolor, bgalpha)
 
         style_provider = Gtk.CssProvider()
-        style_provider.load_from_data(css)
+        style_provider.load_from_data(css.encode('utf-8'))
         self.style_providers.append(style_provider)
 
         # Attempt to load some theme specific stylistic tweaks for appearances
@@ -537,19 +484,20 @@ class Terminator(Borg):
 
         # Size the GtkPaned splitter handle size.
         css = ""
-        if self.config['handle_size'] in xrange(0, 21):
+        if self.config['handle_size'] in range(0, 21):
             css += """
-                .terminator-terminal-window GtkPaned,
-                .terminator-terminal-window paned {
-                    -GtkPaned-handle-size: %s; }
-                """ % self.config['handle_size']
+                .terminator-terminal-window separator {
+                    min-height: %spx;
+                    min-width: %spx; 
+                }
+                """ % (self.config['handle_size'],self.config['handle_size'])
         style_provider = Gtk.CssProvider()
-        style_provider.load_from_data(css)
+        style_provider.load_from_data(css.encode('utf-8'))
         self.style_providers.append(style_provider)
 
         # Apply the providers, incrementing priority so they don't cancel out
         # each other
-        for idx in xrange(0, len(self.style_providers)):
+        for idx in range(0, len(self.style_providers)):
             Gtk.StyleContext.add_provider_for_screen(
                 Gdk.Screen.get_default(),
                 self.style_providers[idx],
